/*-----------------------------------------------------------------------------
Pragma's & Definitions
-----------------------------------------------------------------------------*/
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#define min(a,b)            (((a) < (b)) ? (a) : (b))


/*-----------------------------------------------------------------------------
Includes for Implementation 
-----------------------------------------------------------------------------*/
#include <algorithm>

/*-----------------------------------------------------------------------------

-----------------------------------------------------------------------------*/
inline CPerformanceStatistics::CPerformanceStatistics(const std::string& a_Name)
:TotalTime(0), Samples(0), Name(a_Name), Min(0), Max(0), Percent(1)
{

}


/*-----------------------------------------------------------------------------

-----------------------------------------------------------------------------*/
inline bool CPerformanceStatistics::operator== (const CPerformanceStatistics& a_Stats) const
{
	return Name == a_Stats.Name;
}


/*-----------------------------------------------------------------------------
Locate or create node in tree for this profiling sample session. We don' reset
the timer here as we want this following code included in the profile of this
node, not the parent.
-----------------------------------------------------------------------------*/
inline CPerformanceMonitor::CPerformanceMonitor(const char* a_Name)
{
	if (Statistics.size() == 0)
	{ // The tree is empty, create a root node
		s_Parent = Statistics.insert(Statistics.begin(), CPerformanceStatistics(a_Name));
		m_ThisNode = s_Parent;
	}
	else
	{ // Check to see if we are resampling an existing proile subject.
		CPerformanceStatistics New(a_Name);
		tree<CPerformanceStatistics>::sibling_iterator itr = std::find(Statistics.begin(s_Parent), Statistics.end(s_Parent), New);
		if (itr != Statistics.end(s_Parent))
			m_ThisNode = itr;
		else
		{ //Profile subject did not exist, create a new one.
			s_Parent = Statistics.append_child(s_Parent, New);
			m_ThisNode = s_Parent;
		}
	}
}


/*-----------------------------------------------------------------------------
Close this profile sample session. Append data to sample in tree.
-----------------------------------------------------------------------------*/
inline CPerformanceMonitor::~CPerformanceMonitor()
{
	unsigned int Elapsed = m_Timer.ElapsedTicks();

	// Record statistics for this sampling
	++(m_ThisNode->Samples);
	m_ThisNode->TotalTime += Elapsed;

	if (m_ThisNode->Samples == 1)
	{
		m_ThisNode->Max = Elapsed;
		m_ThisNode->Min = Elapsed;
	}
	else
	{
		m_ThisNode->Max = max(Elapsed, m_ThisNode->Max);
		m_ThisNode->Min = min(Elapsed, m_ThisNode->Min);
	}
	
	// Reset parent pointer
	s_Parent = Statistics.parent(m_ThisNode);
}


/*-----------------------------------------------------------------------------

-----------------------------------------------------------------------------*/
inline void CPerformanceMonitor::Reset(void)
{
	Statistics.clear();
}
